@@ -9,6 +9,12 @@ import os
 import frontmatter
 
 
+class PostNotFound(Exception):
+    """
+    Error when a post isn't where you think it is
+    """
+
+
 class Stack(object):
     """
     A Stack takes a source directory, output directory, optional middleware and metadata
@@ -49,15 +55,18 @@ class Stack(object):
         self.files.update(files)
         return files
 
-    def iter(self, reset=False):
+    def iter(self, reset=False, reverse=False):
         """
         Yield processed files one at a time, in natural order.
         """
         files = os.listdir(self.source)
-        files.sort()
+        files.sort(reverse=reverse)
 
         for filename in files:
-            yield self.get(filename, reset)
+            try:
+                yield self.get(filename, reset)
+            except PostNotFound:
+                continue
 
     def get(self, filename, reset=False):
         """
@@ -82,7 +91,10 @@ class Stack(object):
         self.files.update(files)
 
         # return just the post
-        return files[filename]
+        try:
+            return files[filename]
+        except KeyError:
+            raise PostNotFound('{0} not found'.format(filename))
 
     def build(self, dest=None):
         "Build out results to dest directory (creating if needed)"
@@ -102,7 +114,6 @@ class Stack(object):
             os.makedirs(self.dest)
 
         # make sure we have files
-        #files = getattr(self, 'files', self.run())
         if not self.files:
             self.run()
 
@@ -136,3 +147,22 @@ class Stack(object):
 
         return list(data)
 
+    def use(self, func):
+        """
+        Add function (or other callable) to the middleware stack.
+        Takes a single function as an argument and returns it,
+        so this can be used as a decorator.
+
+        func should take two arguments: files and stack
+
+        @stack.use
+        def count_files(files, stack):
+            stack.metadata['count'] = len(files)
+
+        """
+        if not callable(func):
+            raise TypeError('Stack.use requires a callable')
+
+        self.middleware.append(func)
+        return func
+
